# (c) Copyright 2010-2011. CodeWeavers, Inc.

import os
import subprocess
import sys
import traceback

import cxutils

from cxutils import cxgettext as _

def display_error_zenity(text):
    if isinstance(text, cxutils.unicode_type):
        text = text.encode('utf8')

    subprocess.Popen(['zenity', '--error', '--text', text])

def display_error_kdialog(text):
    if isinstance(text, cxutils.unicode_type):
        text = text.encode('utf8')

    subprocess.Popen(['kdialog', '--error', text])

def display_error_xmessage(text):
    if isinstance(text, cxutils.unicode_type):
        text = text.encode('utf8') # FIXME

    subprocess.Popen(['xmessage', text])

def display_error(text):
    try:
        display_error_zenity(text)
    except:
        try:
            display_error_kdialog(text)
        except:
            try:
                display_error_xmessage(text)
            except:
                traceback.print_exc()

def has_gtk():
    try:
        # pylint: disable-msg=F0401,W0612
        import gtk
        import gtk.glade
    except ImportError:
        return False
    except Exception:
        traceback.print_exc()
    return True

def ensure_present_gtk():
    # Check whether gtk is present, and if not, try to re-exec the current
    # process with a version of python that has it. Return True if gtk is present.
    if has_gtk():
        return True

    for candidate in ('python', 'python2', 'python3'):
        try:
            popen = subprocess.Popen([candidate, __file__])
            if popen.wait() == 0:
                os.execvp(candidate, [candidate] + sys.argv)
        except Exception:
            traceback.print_exc()

    return False

def check_gtk():
    if not ensure_present_gtk():
        display_error(_(
"""You seem to be missing the GTK Python modules.
If this is the case you may try running one of the following commands as root:

Debian / Ubuntu: apt-get install python-gtk2 python-glade2
Fedora:          yum install pygtk2 pygtk2-libglade
Mandriva:        urpmi pygtk2.0 pygtk2.0-libglade
SUSE:            yast -i python-gtk"""))

if __name__ == '__main__':
    if has_gtk():
        sys.exit(0)
    else:
        sys.exit(1)
