# (c) Copyright 2009-2012. CodeWeavers, Inc.

# for localization
from cxutils import cxgettext as _

import os
import traceback

import iso8601

import gtk
import gtk.glade
import gobject
gtk.gdk.threads_init()

import distversion
import cxproduct
import cxutils
import bottlequery
import bottlewrapper
import cxlog
import importdlg
import pyop

import c4parser
import c4profiles
import c4profilesmanager
import installtask
import cxaiebase
import cxaiecore
import cxaiengine
import cxaiemedia

import cxguitools
import bottlecollection
import bottlemanagement
import mountpoints


download_queue = pyop.PythonOperationQueue(maxThreads=2)


def package_select_filter(model, filteriter, wizard):
    isGroup = model.get_value(filteriter, 3)
    if isGroup or model.get_value(filteriter, 1) == 'com.codeweavers.unknown':
        return True
    profile = wizard.installTask.profiles.get(model.get_value(filteriter, 1))
    searchString = wizard.packageSearchString.lower()
    if searchString and profile:
        for lang in cxutils.get_preferred_languages():
            if searchString in profile.localized_names.get(lang, '').lower():
                return True
        else:
            return False
    else:
        return True
    

class InstallerAssistantController(installtask.InstallTaskDelegate):

    media_markup_format = '<b>%s</b>\n<small>%s</small>'
    registered_delegate = False

    def __init__(self, **kwargs):
        installtask.InstallTaskDelegate.__init__(self)
        self.options = kwargs
        self.auto_update = False

        #-----------------------------------------------------------------------
        #  Setup the GUI
        #-----------------------------------------------------------------------

        try:
            self.gladepath = os.environ["CX_GLADEPATH"]
        except KeyError:
            self.gladepath = os.path.join(cxutils.CX_ROOT, "lib", "python", "glade")

        self.gladefile = os.path.join(self.gladepath, "installerassistant.glade")

        gtk.glade.set_custom_handler(cxguitools.CustomWidgets.custom_widget_handler)
        if not gtk.glade.textdomain() == "crossover":
            locale_path = os.path.join(cxutils.CX_ROOT, "share", "locale")
            gtk.glade.bindtextdomain("crossover", locale_path)
            gtk.glade.textdomain("crossover")
        self.xml = gtk.glade.XML(self.gladefile)
        self.xml.signal_autoconnect(self)
        self.animateFunctions = []

        # Restore settings
        global_config = cxproduct.get_config()
        expanded_state = global_config['OfficeSetup'].get('ExpandInstallDetails', '0')
        if expanded_state == '1':
            self.xml.get_widget('InstallerTasksExpander').set_expanded(True)

        self.show_untested_apps = (global_config['OfficeSetup'].get('ShowUntestedApps', '0') != '0')

        self.installTask = installtask.InstallTask(self.show_untested_apps, delegate=self)
        self.installEngine = None
        self.install_in_progress = False
        self.install_ui = False # True if we're doing something in an install that requires user interaction
        self.failing_tasks = []

        self.autorun_file = None

        # Tweaks not allowed by glade.
        install_notes = self.xml.get_widget('InstallationNotes')
        install_notes.set_size_request(1, -1)
        warning_image = install_notes.render_icon(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)
        if warning_image:
            install_notes.images[u'warning'] = warning_image
        error_image = install_notes.render_icon(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_MENU)
        if error_image:
            install_notes.images[u'error'] = error_image

        self.xml.get_widget('PackageDescription').set_size_request(1, -1)

        self.xml.get_widget('DownloadInstallationNotes').set_size_request(1, -1)

        self.xml.get_widget('InstallResults').set_size_request(1, -1)


        # The fields are: label, icon, mountpoint, sort key, markup, temporary
        self.volumesListModel = gtk.ListStore(str, gtk.gdk.Pixbuf, str, str, str, bool)
        self.volumesListModel.set_sort_column_id(3, gtk.SORT_ASCENDING)

        volumesWidget = self.xml.get_widget("MediaTreeView")
        renderer = gtk.CellRendererPixbuf()
        renderer.set_property('xpad', 6)
        renderer.set_property('ypad', 6)
        column = gtk.TreeViewColumn("Icon", renderer, pixbuf = 1)
        volumesWidget.append_column(column)
        column = gtk.TreeViewColumn("Caption", gtk.CellRendererText(), markup = 4)
        volumesWidget.append_column(column)
        volumesWidget.set_model(self.volumesListModel)

        # Add the special 'Browse...' options
        newrow = self.volumesListModel.append()
        self.volumesListModel.set_value(newrow, 0, self.chooseInstallerFileString)
        pixbuf = volumesWidget.render_icon(gtk.STOCK_EXECUTE, gtk.ICON_SIZE_DIALOG)
        self.volumesListModel.set_value(newrow, 1, pixbuf)
        self.volumesListModel.set_value(newrow, 3, '2')
        self.volumesListModel.set_value(newrow, 4, self.media_markup_format % (self.chooseInstallerFileString,
            _("The file you select will be launched to install the application")))

        newrow = self.volumesListModel.append()
        self.volumesListModel.set_value(newrow, 0, self.chooseInstallerFolderString)
        pixbuf = volumesWidget.render_icon(gtk.STOCK_DIRECTORY, gtk.ICON_SIZE_DIALOG)
        self.volumesListModel.set_value(newrow, 1, pixbuf)
        self.volumesListModel.set_value(newrow, 3, '3')
        self.volumesListModel.set_value(newrow, 4, self.media_markup_format % (self.chooseInstallerFolderString,
            _("The folder you select will be treated like an installer disc")))

        bottlesWidget = self.xml.get_widget("BottleSelectionView")
        column = gtk.TreeViewColumn(_("Name"), gtk.CellRendererText(), markup = 1)
        column.set_resizable(True)
        bottlesWidget.append_column(column)
        column = gtk.TreeViewColumn(_("Type"), gtk.CellRendererText(), text = 2)
        column.set_resizable(False)
        bottlesWidget.append_column(column)
        self.bottleListModel = gtk.TreeStore(object, str, str, str) #name, display markup, template, sort key
        self.bottleListModel.set_sort_column_id(3, gtk.SORT_ASCENDING)
        bottlesWidget.set_model(self.bottleListModel)

        # locale name, locale id, sortkey
        self.languageTreeStore = gtk.ListStore(str, str, str)
        self.languageTreeStore.set_sort_column_id(2, gtk.SORT_ASCENDING)

        # The fields are: name, appid, sortkey
        self.packageTreeStore = gtk.TreeStore(str, str, str, bool)
        self.packageTreeModel = self.packageTreeStore.filter_new()
        packageTreeView = self.xml.get_widget("PackageTreeView")
        packageTreeView.set_model(self.packageTreeModel)
        # The column header is not shown so there is no point localizing it
        column = gtk.TreeViewColumn("Installable Packages", gtk.CellRendererText(), markup = 0)
        column.set_resizable(False)
        packageTreeView.append_column(column)
        self.packageTreeStore.set_sort_column_id(2, gtk.SORT_ASCENDING)
        self.packageTreeModel.set_visible_func(package_select_filter, self)
        self.packageSearchString = ""

        collection = bottlecollection.sharedCollection(('basic', 'applications'))
        collection.addChangeDelegate(self)
        collection.addBottleChangeDelegate(self)
        self.registered_delegate = True
        self.bottleCollectionChanged()

        self.show_dialog()

        gobject.timeout_add(100, self.animate)

        update = global_config['OfficeSetup'].get('AutoUpdate', '')

        importbottles = global_config['OfficeSetup'].get('NeverImportGamesBottles', '')
        if importbottles == '' and len(bottlequery.get_games_bottle_list()):
            importdlg.ImportDialogController(parent = self.xml.get_widget('InstallerAssistant'))

        infobar = self.xml.get_widget('PackageInfoBar')

        if update == '':
            infobar.set_message_type(gtk.MESSAGE_QUESTION)
            infobar_label = gtk.Label(_("%s can download updates to this list.") % distversion.PRODUCT_NAME)
            infobar_label.set_alignment(0.0, 0.5)
            infobar_label.show()
            infobar.get_content_area().add(infobar_label)
            infobar.add_button(_("Update Now"), 3)
            infobar.add_button(_("Never Update"), 1)
            infobar.add_button(_("Always Update"), 0)
            close_image = gtk.image_new_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_BUTTON)
            close_image.show()
            close_button = gtk.Button()
            close_button.add(close_image)
            close_button.set_relief(gtk.RELIEF_NONE)
            close_button.show()
            infobar.add_action_widget(close_button, 2)
            infobar.connect('response', self.auto_update_response)
            infobar.show()
        else:
            infobar.hide()


        if update == '1':
            # Start an update of online profiles.
            c4p_url = global_config['OfficeSetup'].get('TieURL', '')
            updateOp =  UpdateProfilesOperation(c4p_url, self)
            pyop.sharedOperationQueue.enqueue(updateOp)
            self.auto_update = True

        # Start parsing any extra package information now
        if self.options.get('c4pfile'):
            self.parse_c4pfile(self.options['c4pfile'])

        self.mountpoints = mountpoints.MountPointsNotifier(self.volumeAdded, self.volumeDeleted)

        # Try to guess the profile from inserted CD's.
        self.installTask.AutoFillSettings()


    def auto_update_response(self, _widget, response):
        if response in (0, 3):
            if response == 0:
                cxproduct.save_setting('OfficeSetup', 'AutoUpdate', '1')
            global_config = cxproduct.get_config()
            c4p_url = global_config['OfficeSetup'].get('TieURL', '')
            updateOp = UpdateProfilesOperation(c4p_url, self)
            pyop.sharedOperationQueue.enqueue(updateOp)
        elif response == 1:
            cxproduct.save_setting('OfficeSetup', 'AutoUpdate', '0')
        self.xml.get_widget('PackageInfoBar').hide()


    def dropin_c4p(self, filename):
        [newfilename, sig] = c4profilesmanager.C4ProfilesSet.prepare_external_c4p(filename, True)
        if not newfilename:
            return
        self.autorun_file = c4profilesmanager.C4ProfilesSet.add_c4p_file(newfilename, sig, c4profiles.dropin)

        if filename:
            os.remove(newfilename)
        if sig:
            os.remove(sig)

        if self.autorun_file.useful_for_current_product:
            self.installTask.update_profiles()
            if self.xml.get_widget('InstallerAssistant').get_property('visible'):
                self.updatePackageListing()
                self.installTask.use_autorun_file(self.autorun_file)
                self.installTask.AutoFillSettings()
        else:
            title = _("Invalid file")
            failstring = _("The file '%s' does not contain installation profiles for anything that can be installed with this product.") % filename
            cxguitools.CXMessageDlg(primary=title, secondary=failstring,
                                    message_type=gtk.MESSAGE_WARNING,
                                    parent=self.xml.get_widget('InstallerAssistant'))

    def parseResponse(self, response, userdata):
        if response == 0:
            self.dropin_c4p(userdata)

    def is_c4pfile_trusted(self, c4pfile):
        # at present all external c4pfiles are treated as untrusted
        return c4profilesmanager.C4ProfilesSet.taste_new_c4p_file(c4pfile)

    def parse_c4pfile(self, filename):
        c4pfile = c4parser.C4PFile(filename, False, None)
        if c4pfile.malware_appid:
            cxguitools.show_malware_dialog(c4pfile, parent=self.xml.get_widget('InstallerAssistant'))
        elif not self.is_c4pfile_trusted(filename):
            primary = _("The file '%(filename)s' is untrusted. Are you sure you want to load it?") % {'filename' : filename}
            secondary = _("This file is not from a trusted source!\n\nThis means that CodeWeavers has no knowledge or control over what actions this CrossTie file may attempt. Be VERY cautious loading untrusted CrossTies as they can cause great damage to your computer. CodeWeavers will not provide technical support for issues related to the use of untrusted CrossTie files.")
            cxguitools.CXMessageDlg(secondary=secondary, primary=primary,
                                        button_array=[[_("Don't Load"), 1],
                                                      [_("Load"), 0]],
                                        user_data=filename,
                                        response_function=self.parseResponse,
                                        message_type=gtk.MESSAGE_WARNING)
        else:
            self.dropin_c4p(filename)

    def cancel_clicked(self, _caller):
        if self.install_in_progress:
            self.cancel_install()
        else:
            self.destroy()

    def PackageRefreshButton_clicked(self, _caller):
        global_config = cxproduct.get_config()
        c4p_url = global_config['OfficeSetup'].get('TieURL', '')
        updateOp =  UpdateProfilesOperation(c4p_url, self)
        pyop.sharedOperationQueue.enqueue(updateOp)

    def destroy(self):
        if self.registered_delegate:
            collection = bottlecollection.sharedCollection()
            collection.removeChangeDelegate(self)
            collection.removeBottleChangeDelegate(self)
            self.registered_delegate = False
        if self.xml.get_widget('InstallerTasksExpander').get_expanded():
            expanded = '1'
        else:
            expanded = '0'
        cxproduct.save_setting('OfficeSetup', 'ExpandInstallDetails', expanded)
        self.xml.get_widget("InstallerAssistant").destroy()
        self.quit_requested(None)

    def quit_requested(self, _caller):
        cxguitools.toplevel_quit()

    def close_clicked(self, _arg1, _arg2):
        if not self.install_in_progress:
            self.destroy()
            return False
        else:
            primary = _("Abort the installation?")
            secondary = _("Closing this window will interrupt the installation and leave your software partially installed. Do you really want to do this?")
            cxguitools.CXMessageDlg(secondary=secondary, primary=primary,
                                        button_array=[[_("Continue Installation"), 0],
                                                      [_("Close"), 1]],
                                        response_function=self.close_dlg_response,
                                        message_type = gtk.MESSAGE_WARNING)
            return True

    def close_dlg_response(self, response):
        if response == 1:
            self.cancel_install()
            self.destroy()

    def show_dialog(self):
        dialog = self.xml.get_widget("InstallerAssistant")
        self.updatePackageListing()
        self.profileChanged()
        self.updateBottleListing()
        self.sourceChanged()
        self.bottleCreateChanged()
        self.bottleNewnameChanged()
        self.bottleTemplateChanged()

        if self.options.get('bottle') and self.options['bottle'] in bottlecollection.sharedCollection().bottleList():
            self.installTask.bottlename = self.options['bottle']

        if self.options.get('installersource'):
            self.set_custom_installer_source(self.options['installersource'])

        if self.options.get('profileid') and self.options['profileid'] in self.installTask.profiles:
            self.installTask.profile = self.installTask.profiles[self.options['profileid']]

        if self.autorun_file:
            self.installTask.use_autorun_file(self.autorun_file)

        self.set_expanded_section('PackageSelect')
        dialog.show()

    def set_expanded_section(self, name):
        for section_name in ['PackageSelect', 'MediaSelect', 'BottleSelect', 'Installer', 'Finished']:
            if section_name == name:
                self.xml.get_widget('%sArrow' % section_name).set(gtk.ARROW_DOWN, gtk.SHADOW_NONE)
                self.xml.get_widget('%sContent' % section_name).show()
            else:
                self.xml.get_widget('%sArrow' % section_name).set(gtk.ARROW_RIGHT, gtk.SHADOW_NONE)
                self.xml.get_widget('%sContent' % section_name).hide()
        self.on_section_changed[name](self)

    def advance_section(self, can_install=False):
        # Expand the first section that needs work.
        # If can_install is True and no sections need work, start the install.
        if self.installEngine:
            # Install has started already.
            self.destroy()
        elif self.wantPackageSelection():
            self.set_expanded_section('PackageSelect')
        elif self.wantMediaSelection():
            self.set_expanded_section('MediaSelect')
        elif self.wantBottleSelection():
            self.set_expanded_section('BottleSelect')
        elif can_install and self.installTask.can_install():
            self.displayInstallationProgressPanel()
            self.doInstall()
        else:
            self.set_expanded_section(None)

    def next_clicked(self, _widget):
        self.advance_section(True)

    def update_common_widgets(self):
        # update next button
        nextButton = self.xml.get_widget("InstallerNext")
        if (self.xml.get_widget('PackageSelectContent').get_property('visible') and self.wantPackageSelection()) or \
                (self.xml.get_widget('MediaSelectContent').get_property('visible') and self.wantMediaSelection()) or \
                (self.xml.get_widget('BottleSelectContent').get_property('visible') and self.wantBottleSelection()):
            nextButton.set_label(_("Proceed"))
            nextButton.set_sensitive(False)
        elif self.wantPackageSelection() or self.wantMediaSelection() or self.wantBottleSelection():
            nextButton.set_label(_("Proceed"))
            nextButton.set_sensitive(True)
        elif not self.installTask.can_install():
            # Can't install but no sections need work. Install notes should say why.
            nextButton.set_label(_("Proceed"))
            nextButton.set_sensitive(False)
        else:
            nextButton.set_label(_("Install"))
            nextButton.set_sensitive(True)

        self.update_install_notes(True)

    def get_install_notes(self, show_warnings, escape=False):
        installationNotesText = []

        warning_icon_html = '<img src="warning" alt="WARNING" />'
        error_icon_html = '<img src="error" alt="ERROR" />'

        if self.installTask.profile is None:
            installationNotesText.append(_("Select the application to install from the list above, or \"Other Application\" if it's not listed."))
        elif self.installTask.installationNotes:
            if escape:
                # Assume the original HTML markup is broken so sanitize it
                notes = cxutils.html_to_text(self.installTask.installationNotes)
                notes = cxutils.html_escape(notes)
                # Only preserve line feeds
                notes = notes.replace('\n', '<br/>')
                installationNotesText.append(notes)
            else:
                installationNotesText.append(self.installTask.installationNotes)

        if show_warnings:
            if self.installTask.target_bottle and \
                    self.installTask.target_bottle.analyzed():
                for warning in self.installTask.target_bottle.warnings:
                    installationNotesText.append(warning_icon_html + " " + warning)

            # cxdiag messages
            for key, (level, title, description) in self.installTask.get_cxdiag_messages().iteritems():
                if level == 'Require':
                    installationNotesText.append('%s <a href="http://www.codeweavers.com/support/wiki/Diag/%s">%s</a>: %s' % (warning_icon_html, key, title, description))

            # check that the new bottle name doesn't match an existing bottle
            if self.installTask.GetCreateNewBottle() and self.installTask.bottlename:
                for bottlename in bottlecollection.sharedCollection().bottleList():
                    if self.installTask.bottlename.lower() == bottlename.lower():
                        installationNotesText.append(_("%(error_icon)s There is already a bottle named '%(bottlename)s'. Select the existing bottle from the list or choose a different bottle name.") % {
                            'error_icon': error_icon_html,
                            'bottlename': self.installTask.bottlename})
                        break

            if not self.wantPackageSelection() and not self.wantMediaSelection() and not self.wantBottleSelection():
                # We have a greyed out Install button
                if not self.installTask.target_bottle.analyzed():
                    installationNotesText.append(_("The install cannot continue until the '%s' bottle is scanned for dependencies. Please wait.") % self.installTask.bottlename)

            if self.installTask.target_bottle and self.installTask.target_bottle.analyzed():
                if self.installTask.target_bottle.missing_profiles:
                    missing = ", ".join(self.installTask.target_bottle.missing_profiles)
                    installationNotesText.append(error_icon_html + u' ' + _("The install cannot continue because one or more profiles it depends on could not be found: %s") % missing)
                elif self.installTask.target_bottle.missing_media:
                    missing = ", ".join(self.installTask.get_profile_names(self.installTask.target_bottle.missing_media))
                    installationNotesText.append(error_icon_html + u' ' + _("The install cannot continue because no installer could be found for the following dependencies: %s") % missing)
                else:
                    deps = self.installTask.get_dependencies()
                    if deps:
                        deps_string = ', '.join(deps)
                        installationNotesText.append(_("These software packages will also be installed: %s") % deps_string)

        return '<body>%s</body>' % (''.join('<p>%s</p>' % x for x in installationNotesText))

    def update_install_notes(self, show_warnings):
        installationNotesTextView = self.xml.get_widget("InstallationNotes")

        htmlNotes = self.get_install_notes(show_warnings)

        if htmlNotes:
            installationNotesTextView.get_buffer().set_text("")
            try:
                installationNotesTextView.display_html(htmlNotes.encode("utf8"))
            except:
                # Obtain a sanitized version
                htmlNotes = self.get_install_notes(show_warnings, escape=True)
                installationNotesTextView.display_html(htmlNotes.encode("utf8"))

            installationNotesTextView.show()
        else:
            installationNotesTextView.hide()


    #-----------------------------------------------------------------------
    #  Package Selection
    #-----------------------------------------------------------------------
    def wantPackageSelection(self):
        return self.installTask.profile is None

    def package_toggle(self, _widget):
        if self.xml.get_widget('PackageSelectContent').get_property('visible'):
            self.advance_section()
        else:
            self.set_expanded_section('PackageSelect')

    matchingString = "<b>"+_("Ready to Install")+"</b>"
    communitySupportedString = "<b>"+_("Community Supported Applications")+"</b>"
    unsupportedString = "<b>"+_("Unsupported Applications")+"</b>"
    supportedString = "<b>"+_("Supported Applications")+"</b>"
    componentString = "<b>"+_("Runtime Support Components")+"</b>"
    upgradeString = "<b>"+_("Service Packs and Upgrades")+"</b>"

    def updateMatchingProfiles(self):
        matchingProfiles = self.installTask.profiles_matching_media
        nodeIter = None

        thisNode = self.packageTreeStore.get_iter_first()
        while thisNode:
            if self.packageTreeStore.get_value(thisNode, 0) == self.matchingString:
                nodeIter = thisNode
                break
            thisNode = self.packageTreeStore.iter_next(thisNode)

        if nodeIter:
            self.packageTreeStore.remove(nodeIter)

        if len(matchingProfiles):
            nodeIter = self.packageTreeStore.append(None, [self.matchingString, '.matching', "zzzz0 " + self.matchingString, True])
            for appid in matchingProfiles:
                profile = self.installTask.profiles[appid]
                if not profile.is_for_current_product:
                    continue

                if not profile.is_ranked and not self.show_untested_apps:
                    continue

                # These will be in the list more than once -- no problem there.
                self.packageTreeStore.append(nodeIter, [cxutils.html_escape(profile.name), appid, profile.name, False])

                path = self.packageTreeStore.get_path(nodeIter)
                self.xml.get_widget("PackageTreeView").expand_to_path(path)
                self.xml.get_widget("PackageTreeView").scroll_to_cell(path)


        self.xml.get_widget("PackageTreeView").queue_draw()


    def updatePackageListing(self):
        packageTreeView = self.xml.get_widget("PackageTreeView")
        self.packageTreeStore.clear()

        supportedParent = self.packageTreeStore.append(None, [self.supportedString, '.supported', "zzzz1 " + self.supportedString, True])
        unsupportedParent = self.packageTreeStore.append(None, [self.unsupportedString, '.unsupported', "zzzzz3 " + self.unsupportedString, True])
        communityParent = self.packageTreeStore.append(unsupportedParent, [self.communitySupportedString, '.communitysupported', self.communitySupportedString, True])
        upgradeParent = self.packageTreeStore.append(None, [self.upgradeString, '.upgrade', "zzzzz4 " + self.upgradeString, True])
        componentParent = self.packageTreeStore.append(None, [self.componentString, '.component', "zzzzz5 " + self.componentString, True])
        for appid, profile in self.installTask.profiles.iteritems():
            if not profile.is_for_current_product:
                continue

            if not profile.is_ranked and not self.show_untested_apps and \
                    appid != c4profilesmanager.UNKNOWN:
                continue

            # FIXME: If app_profile.extra_fors matches one of the installed
            # applications, then put it at the top of the list.
            if profile.app_profile and profile.app_profile.category == "Component/Service Pack":
                self.packageTreeStore.append(upgradeParent, [cxutils.html_escape(profile.name), appid, profile.name, False])
            else:
                if appid == c4profilesmanager.UNKNOWN:
                    self.packageTreeStore.append(unsupportedParent, [cxutils.html_escape(profile.name), appid, profile.name, False])
                elif profile.app_profile.is_supported:
                    if profile.is_application:
                        self.packageTreeStore.prepend(supportedParent, [cxutils.html_escape(profile.name), appid, profile.name, False])
                    if profile.is_component:
                        self.packageTreeStore.append(componentParent, [cxutils.html_escape(profile.name), appid, profile.name, False])
                    if not profile.is_component and not profile.is_application:
                        self.packageTreeStore.append(supportedParent, [cxutils.html_escape(profile.name), appid, profile.name, False])
                else:
                    self.packageTreeStore.append(communityParent, [cxutils.html_escape(profile.name), appid, profile.name, False])

        self.updateMatchingProfiles()

        packageTreeView.expand_row(0, False)
        packageTreeView.expand_row(1, False)

    def package_changed_gui(self, _widget):
        packageTreeView = self.xml.get_widget("PackageTreeView")
        cursor = packageTreeView.get_cursor()
        if cursor and cursor[0]:
            is_category = self.packageTreeModel.get_value(self.packageTreeModel.get_iter(cursor[0]), 3)
            if not is_category:
                selectedID = self.packageTreeModel.get_value(self.packageTreeModel.get_iter(cursor[0]), 1)
                self.installTask.profile = self.installTask.profiles[selectedID]
                if cxlog.is_on('c4profiles'):
                    cxlog.log_('c4profiles', "selected %s" % cxlog.to_str(self.installTask.profile.appid))
                    self.installTask.profile.dump(cxlog.get_file(), "| ")

    def package_row_activated(self, _widget, path, _column):
        packageTreeView = self.xml.get_widget("PackageTreeView")
        is_category = self.packageTreeModel.get_value(self.packageTreeModel.get_iter(path), 3)
        if is_category:
            if packageTreeView.row_expanded(path):
                packageTreeView.collapse_row(path)
            else:
                packageTreeView.expand_row(path, False)
        else:
            self.advance_section(False)

    def select_package_by_appid(self, model, path, treeiter, appid):
        if model.get_value(treeiter, 1) == appid:
            treeview = self.xml.get_widget("PackageTreeView")
            treeview.expand_to_path(path)
            treeview.set_cursor(path)
            treeview.scroll_to_cell(path)
            return True # stop iteration
        else:
            return False # continue iteration

    def select_language_by_id(self, model, path, treeiter, langid):
        if model.get_value(treeiter, 1) == langid:
            self.xml.get_widget("LanguageSelectorCombo").set_active(path[0])
            return True # stop iteration
        else:
            return False # continue iteration

    medal_names = {
        'gold': _("Gold Medal (Officially Supported)"),
        'silver': _("Silver Medal (Officially Supported)"),
        'bronze': _("Bronze Medal (Officially Supported)"),
        'ungold': _("Gold Medal (Unsupported)"),
        'unsilver': _("Silver Medal (Unsupported)"),
        'unbronze': _("Bronze Medal (Unsupported)"),
        'knownnottowork': _("Known not to work"),
        'untested': _("Untested"),
        'untrusted': _("Untrusted"),
    }

    medal_descriptions = {
        'gold': _("Installs and runs as you would expect in Microsoft Windows. We expect that you can use this application on an everyday basis with good results and only minor bugs."),
        'silver': _("Installs and runs well enough to be usable. However, this application has bugs that prevent it from running flawlessly."),
        'bronze': _("Installs, runs, and can accomplish some portion of its fundamental mission. However, this application has enough bugs that we recommend you use it with caution. Save early/save often, and don't be surprised if there are some bumps along the way."),
        'ungold': _("Installs and runs as you would expect in Microsoft Windows. Our users expect that you can use this application on an everyday basis with good results and only minor bugs."),
        'unsilver': _("Installs and runs well enough to be usable. However, our users find that this application has bugs that prevent it from running flawlessly."),
        'unbronze': _("Installs, runs, and can accomplish some portion of its fundamental mission. However, this application has enough bugs that we recommend you use it with caution. Save early/save often, and don't be surprised if there are some bumps along the way."),
        'knownnottowork': _("This application either does not install or does not run."),
        'untested': _("We do not know whether or not this application will install and run. If you try it, consider letting us know how it runs, using the compatibility database."),
        'untrusted': _("This profile is from an unknown source. Installing may pose a security risk to your system."),
    }

    def update_profile_caption(self):
        ToggleLabel = self.xml.get_widget("PackageSelectLabel")
        if self.installTask.installerLocale:
            langid = self.installTask.installerLocale
            ToggleLabel.set_text(_("Will install %(application)s (%(language)s version)") % {
                'application': self.installTask.profile.name,
                'language': c4profiles.LANGUAGES.get(langid, langid)})
        else:
            ToggleLabel.set_text(_("Will install %s") % self.installTask.profile.name)

    def profileChanged(self):
        supportedLabel = self.xml.get_widget("PackageSupportedLabel")
        medalImage = self.xml.get_widget("MedalImage")
        descriptionTextView = self.xml.get_widget("PackageDescription")
        ToggleLabel = self.xml.get_widget("PackageSelectLabel")
        MediaSelectToggle = self.xml.get_widget("MediaSelectToggle")
        packageTreeView = self.xml.get_widget("PackageTreeView")

        if self.installTask.profile:
            description_lines = []

            cursor = packageTreeView.get_cursor()

            if not (cursor and cursor[0] and
                    self.packageTreeModel.get_value(self.packageTreeModel.get_iter(cursor[0]), 1) == self.installTask.profile.appid):
                # selection changed
                self.packageTreeModel.foreach(self.select_package_by_appid, self.installTask.profile.appid)

            medal = self.installTask.profile.app_profile.medal

            # Show medal data
            if self.installTask.profile.app_profile.is_supported:
                markup = _("Supported by CodeWeavers")
            else:
                markup = _("Not Supported by CodeWeavers")
            markup = "<b>" + markup + "</b>"

            if medal:
                try:
                    date = iso8601.parse_date(self.installTask.profile.app_profile.medal_date)
                    markup += "\n" + _("Last Ranked: ") + date.strftime("%x")
                except:
                    # Just don't show a date
                    pass

            supportedLabel.set_markup(markup)

            # Show medal icon
            if medal:
                medalImage.set_from_file(self.installTask.profile.app_profile.medal_icon)
                markup = '<b>%s</b>\n%s' % (self.medal_names[medal], self.medal_descriptions[medal])
            else:
                medalImage.set_from_file(None)
                markup = ''

            if hasattr(medalImage, 'set_tooltip_markup'):
                # GTK >= 2.12
                medalImage.set_tooltip_markup(markup)

            description_lines.append('%s' % self.installTask.profile.app_profile.description)

            if self.installTask.profile.details_url:
                description_lines.append('<a href="%s">%s</a>' % (
                    cxutils.html_escape(self.installTask.profile.details_url),
                    cxutils.html_escape(_("More info"))))

            if self.installTask.profile.contributors:
                description_lines.append(_("Tie contributors: %s") % self.installTask.profile.contributors_html.decode('utf8'))

            description = '<body>%s</body>' % ''.join('<p>%s</p>' % line for line in description_lines)

            descriptionTextView.get_buffer().set_text("")
            try:
                #  Encode description as utf8 because htmltextview likes that better.
                descriptionTextView.display_html(description.encode('utf8'))
            except:
                cxlog.log("Display_html is failing to render the string %s. Probably a malformed string is being displayed." % cxlog.debug_str(description))
                descriptionTextView.get_buffer().set_text(cxutils.html_to_text(self.installTask.profile.app_profile.description))

            self.update_profile_caption()

        else:
            packageTreeView = self.xml.get_widget("PackageTreeView")
            packageTreeView.get_selection().unselect_all()
            supportedLabel.set_markup("")
            medalImage.set_from_file(None)
            descriptionTextView.get_buffer().set_text("")
            ToggleLabel.set_text(_("Select an application to install"))

        languages = self.installTask.profile_languages()
        if languages:
            self.xml.get_widget("LanguageSelectorCombo").set_model(None)
            self.languageTreeStore.clear()
            for langid in languages:
                if langid == '':
                    langname = _("Other language")
                    langsort = '1'
                else:
                    langname = c4profiles.LANGUAGES.get(langid, langid)
                    langsort = u'0'+langname
                self.languageTreeStore.append((langname, langid, langsort))
            self.xml.get_widget("LanguageSelectorCombo").set_model(self.languageTreeStore)
            self.xml.get_widget("LanguageSelectorBox").show()
            self.languageTreeStore.foreach(self.select_language_by_id, self.installTask.installerLocale)
        else:
            self.xml.get_widget("LanguageSelectorBox").hide()
            self.update_download_item()
            self.update_steam_item()

        if self.installTask.virtual:
            MediaSelectToggle.hide()
        else:
            MediaSelectToggle.show()

        self.updateBottleListing()

        self.updateMediaListing()
        self.update_common_widgets()


    def count_packages(self, model, root_path, treeiter, result=None):
        if result is None:
            result = {}

        count = 0

        while treeiter is not None:
            is_category = model.get_value(treeiter, 3)

            if is_category:
                _result, subcount = self.count_packages(model, model.get_path(treeiter), model.iter_children(treeiter), result)
                count += subcount
            else:
                count += 1

            treeiter = model.iter_next(treeiter)

        result[root_path] = count

        return result, count

    def package_select_search_changed(self, _caller):
        searchWidget = self.xml.get_widget("PackageSelectSearch")
        self.packageSearchString = searchWidget.get_text()
        self.packageTreeModel.refilter()

        package_counts, _root_count = self.count_packages(self.packageTreeModel, (), self.packageTreeModel.get_iter_first())

        # Check whether there's 1 category that's not the root or unsupported
        # that has items, and expand it if so.
        categories_with_items = []
        for root_path in package_counts:
            if root_path != ():
                treeiter = self.packageTreeModel.get_iter(root_path)
                category_id = self.packageTreeModel.get_value(treeiter, 1)
                if category_id != '.unsupported' and package_counts[root_path]:
                    categories_with_items.append(root_path)

        if len(categories_with_items) == 1:
            self.xml.get_widget('PackageTreeView').expand_row(categories_with_items[0], False)

    def locale_changed(self, _widget):
        index = self.xml.get_widget('LanguageSelectorCombo').get_active()
        treeiter = self.languageTreeStore.get_iter((index,))
        langid = self.languageTreeStore.get_value(treeiter, 1)
        self.installTask.SetLocale(langid)

        self.update_download_item()
        self.update_steam_item()
        self.update_profile_caption()
        self.update_common_widgets()

    #-----------------------------------------------------------------------
    #  Installer Media Selection
    #-----------------------------------------------------------------------

    def browse_for_installer_file(self, caller):
        filePicker = gtk.FileChooserDialog(_("Choose a Windows installer file"), caller, gtk.FILE_CHOOSER_ACTION_OPEN)

        cxguitools.add_filters(filePicker, cxguitools.FILTERS_INSTALLABLE)
        filePicker.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        filePicker.add_button(gtk.STOCK_OPEN, gtk.RESPONSE_OK)

        if filePicker.run() == gtk.RESPONSE_OK:
            selectedFile = filePicker.get_filename()
        else:
            selectedFile = None

        filePicker.destroy()

        return selectedFile


    def browse_for_installer_dir(self, caller):
        filePicker = gtk.FileChooserDialog(_("Choose the directory containing your installer"), caller, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)

        filePicker.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        filePicker.add_button(gtk.STOCK_OPEN, gtk.RESPONSE_OK)

        if filePicker.run() == gtk.RESPONSE_OK:
            selectedDir = filePicker.get_filename()
        else:
            selectedDir = None

        filePicker.destroy()

        return selectedDir


    def volumeAdded(self, volume, temporary=False):
        if volume.mountpoint == "/" or not volume.is_disc:
            # unlikely that the user wants to use this.
            return

        # We need to check this is not a duplicate because volumeAdded() is
        # also used for the user-selected installers and directories
        volumesIter = self.volumesListModel.iter_children(None)
        while volumesIter != None:
            if self.volumesListModel.get_value(volumesIter, 2) == volume.mountpoint:
                # This is a duplicate
                if not temporary:
                    # Just in case this was added by installtask first
                    self.volumesListModel.set_value(volumesIter, 5, False)
                return
            volumesIter = self.volumesListModel.iter_next(volumesIter)

        if volume.label:
            label = volume.label
        else:
            label = os.path.basename(volume.mountpoint)
        volumesWidget = self.xml.get_widget("MediaTreeView")
        self.installTask.add_source_media(volume.mountpoint, volume.label, volume.device)
        self.updateMatchingProfiles()

        newrow = self.volumesListModel.append()
        self.volumesListModel.set_value(newrow, 0, label)
        self.volumesListModel.set_value(newrow, 2, volume.mountpoint)
        if volume.is_disc:
            pixbuf = volumesWidget.render_icon(gtk.STOCK_CDROM, gtk.ICON_SIZE_DIALOG)
        else:
            pixbuf = volumesWidget.render_icon(gtk.STOCK_HARDDISK, gtk.ICON_SIZE_DIALOG)
        self.volumesListModel.set_value(newrow, 1, pixbuf)
        self.volumesListModel.set_value(newrow, 3, '1'+self.volumesListModel.get_value(newrow, 0))
        self.volumesListModel.set_value(newrow, 4, self.media_markup_format % (label, volume.mountpoint))
        self.volumesListModel.set_value(newrow, 5, temporary)


    def volumeDeleted(self, volume, temporary=False):
        volumesIter = self.volumesListModel.iter_children(None)
        while volumesIter != None:
            if self.volumesListModel.get_value(volumesIter, 2) == volume.mountpoint and \
               (not temporary or self.volumesListModel.get_value(volumesIter, 5)):
                self.volumesListModel.remove(volumesIter)
                self.installTask.remove_source_media(volume.mountpoint)
                if self.installTask.GetInstallerSource() == volume.mountpoint:
                    self.installTask.ClearInstallerSource()
                break
            else:
                volumesIter = self.volumesListModel.iter_next(volumesIter)
        else:
            # currentVolumeLabel not found
            cxlog.log("Warning: removed volume does not exist: %s" % cxlog.debug_str(volume))

        self.updateMatchingProfiles()

    def profileMediaAdded_(self, filename):
        volume = mountpoints.Volume()
        volume.mountpoint = filename
        volume.is_disc = True # Lie so it's taken into account
        self.volumeAdded(volume, True)

    def profileMediaRemoved_(self, filename):
        volume = mountpoints.Volume()
        volume.mountpoint = filename
        volume.is_disc = True # Lie so it's taken into account
        self.volumeDeleted(volume, True)


    def wantMediaSelection(self):
        return not (self.installTask.GetInstallerSource() or
                    self.installTask.installerDownloadSource or
                    self.installTask.virtual or
                    self.installTask.installWithSteam)

    def media_toggle(self, _widget):
        if self.xml.get_widget('MediaSelectContent').get_property('visible'):
            self.advance_section()
        else:
            self.set_expanded_section('MediaSelect')


    chooseInstallerFileString = _(u"Choose Installer File\u2026")
    chooseInstallerFolderString = _(u"Choose Installer Folder\u2026")
    downloadInstallerString = _("Download installer")
    steamInstallerString = _("Install Via Steam")
    def updateMediaListing(self):
        app_profile = self.installTask.profile and self.installTask.profile.app_profile
        downloadNotes = self.xml.get_widget("DownloadInstallationNotes")

        if app_profile and app_profile.download_page_urls:
            if app_profile.download_page_urls:
                _lang, url = cxutils.get_language_value(app_profile.download_page_urls)
                note = _('This software can be downloaded from its <a href="%(url)s">home page</a>.') % {'url': cxutils.html_escape(url)}

                fullNote = "<body> %s </body>" % note
                downloadNotes.get_buffer().set_text("")
                try:
                    downloadNotes.display_html(fullNote.encode('utf8'))
                except:
                    cxlog.warn("Display_html is failing to render the string %s.\n%s." % (cxlog.debug_str(fullNote), traceback.format_exc()))
                    downloadNotes.get_buffer().set_text(fullNote)
                downloadNotes.show()
            else:
                downloadNotes.hide()
        else:
            downloadNotes.hide()

    
    def select_media_by_mountpoint(self, model, path, treeiter, mountpoint):
        if model.get_value(treeiter, 2) == mountpoint:
            self.xml.get_widget("MediaTreeView").set_cursor(path)
            return True # stop iteration
        else:
            return False # continue iteration

    def select_download(self, model, path, treeiter, _userdata):
        if model.get_value(treeiter, 0) == self.downloadInstallerString:
            self.xml.get_widget("MediaTreeView").set_cursor(path)
            return True # stop iteration
        else:
            return False # continue iteration

    def select_steam(self, model, path, treeiter, _userdata):
        if model.get_value(treeiter, 0) == self.steamInstallerString:
            self.xml.get_widget("MediaTreeView").set_cursor(path)
            return True # stop iteration
        else:
            return False # continue iteration

    def sourceChanged(self):
        ToggleLabel = self.xml.get_widget("MediaSelectLabel")

        if self.installTask.installWithSteam:
            self.volumesListModel.foreach(self.select_steam, self.installTask.installerSource)

            ToggleLabel.set_label(_("Will install via Steam"))
        elif self.installTask.installerSource:
            self.volumesListModel.foreach(self.select_media_by_mountpoint, self.installTask.installerSource)

            ToggleLabel.set_label(_("Will install from %s") % self.installTask.installerSource)

        elif self.installTask.installerDownloadSource:
            self.volumesListModel.foreach(self.select_download, self.installTask.installerSource)

            ToggleLabel.set_label(_("Will download the installer from %s") % self.installTask.installerDownloadSource)

        else:
            self.xml.get_widget("MediaTreeView").get_selection().unselect_all()

            ToggleLabel.set_label(_("Select an installer"))

        self.update_common_widgets()

    def update_download_item(self):
        url = self.installTask.GetDownloadURL()

        volumesIter = self.volumesListModel.iter_children(None)
        while volumesIter != None:
            if self.volumesListModel.get_value(volumesIter, 0) == self.downloadInstallerString:
                if url:
                    break
                else:
                    self.volumesListModel.remove(volumesIter)
                    return
            volumesIter = self.volumesListModel.iter_next(volumesIter)
        else:
            if url:
                # No download item; add it.
                volumesIter = self.volumesListModel.append()
                self.volumesListModel.set_value(volumesIter, 0, self.downloadInstallerString)
                pixbuf = self.xml.get_widget('MediaTreeView').render_icon(gtk.STOCK_NETWORK, gtk.ICON_SIZE_DIALOG)
                self.volumesListModel.set_value(volumesIter, 1, pixbuf)
                self.volumesListModel.set_value(volumesIter, 3, '0')
                if self.installTask.installerDownloadSource:
                    self.xml.get_widget("MediaTreeView").set_cursor(self.volumesListModel.get_path(volumesIter))
            else:
                return
        self.volumesListModel.set_value(volumesIter, 4, self.media_markup_format % (self.downloadInstallerString, url))

    def update_steam_item(self):
        volumesIter = self.volumesListModel.iter_children(None)
        while volumesIter != None:
            if self.volumesListModel.get_value(volumesIter, 0) == self.steamInstallerString:
                if self.installTask.GetSteamID():
                    break
                else:
                    self.volumesListModel.remove(volumesIter)
                    return
            volumesIter = self.volumesListModel.iter_next(volumesIter)
        else:
            if self.installTask.GetSteamID():
                volumesIter = self.volumesListModel.append()
                pixbuf = gtk.gdk.pixbuf_new_from_file(os.path.join(cxutils.CX_ROOT,'share/images/steam_logo.png'))
                self.volumesListModel.set_value(volumesIter, 1, pixbuf)
                self.volumesListModel.set_value(volumesIter, 3, '0')
                self.xml.get_widget("MediaTreeView").set_cursor(self.volumesListModel.get_path(volumesIter))
                self.volumesListModel.set_value(volumesIter, 0, self.steamInstallerString)
                self.volumesListModel.set_value(volumesIter, 4, self.media_markup_format % (self.steamInstallerString, ""))
                if self.installTask.installWithSteam:
                    self.xml.get_widget("MediaTreeView").set_cursor(self.volumesListModel.get_path(volumesIter))


    def set_custom_installer_source(self, path):
        volume = mountpoints.Volume()
        volume.mountpoint = path
        volume.is_disc = True # Lie so it's taken into account
        self.volumeAdded(volume)
        self.installTask.SetInstallerSource(path)
        self.update_common_widgets()

    def source_changed_gui(self, _widget):
        mediaTreeView = self.xml.get_widget("MediaTreeView")
        selections = mediaTreeView.get_selection().get_selected_rows()[1]

        if selections:
            selectedPath = selections[0]
            selectedIter = self.volumesListModel.get_iter(selectedPath)
            volumeName = self.volumesListModel.get_value(selectedIter, 0)
            mountPoint = self.volumesListModel.get_value(selectedIter, 2)

            if volumeName == self.chooseInstallerFileString:
                selectedFile = self.browse_for_installer_file(self.xml.get_widget("InstallerAssistant"))
                if selectedFile:
                    self.set_custom_installer_source(selectedFile)
                else:
                    self.installTask.ClearInstallerSource()
                    mediaTreeView.get_selection().unselect_all()
            elif volumeName == self.chooseInstallerFolderString:
                selectedDir = self.browse_for_installer_dir(self.xml.get_widget("InstallerAssistant"))
                if selectedDir:
                    self.set_custom_installer_source(selectedDir)
                else:
                    self.installTask.ClearInstallerSource()
                    mediaTreeView.get_selection().unselect_all()
            elif volumeName == self.downloadInstallerString:
                self.installTask.SetInstallerDownload()
            elif volumeName == self.steamInstallerString:
                self.installTask.SetInstallerSteam()
            else:
                self.installTask.SetInstallerSource(mountPoint)
        else:
            self.installTask.ClearInstallerSource()

        self.update_common_widgets()

    #-----------------------------------------------------------------------
    #  Bottle Selection
    #-----------------------------------------------------------------------

    category_markup = {
        installtask.CAT_NONE: "<b>"+_("Bottles")+"</b>",
        installtask.CAT_RECOMMENDED: "<b>"+_("Recommended Bottles")+"</b>",
        installtask.CAT_COMPATIBLE: "<b>"+_("Compatible Bottles")+"</b>",
        installtask.CAT_INCOMPATIBLE: "<b>"+_("Incompatible Bottles")+"</b>"
        }

    category_sort = {
        installtask.CAT_NONE: '0',
        installtask.CAT_RECOMMENDED: '1',
        installtask.CAT_COMPATIBLE: '2',
        installtask.CAT_INCOMPATIBLE: '3',
        }

    def categorizedBottle_(self, _target_bottle):
        self.updateBottleListing()

    def categorizedAllBottles(self):
        pass

    def analyzedBottle_(self, target_bottle):
        if target_bottle is self.installTask.target_bottle:
            self.update_common_widgets()

    def invalidNewBottleName(self):
        cxlog.log("invalid new bottle name [%s]" % cxlog.to_str(self.installTask.newBottleName))
        # FIXME: Todo

    def wantBottleSelection(self):
        if not self.installTask.bottlename:
            # no bottle selected
            return True
        elif self.installTask.GetCreateNewBottle():
            # check that the name doesn't match an existing bottle
            for bottlename in bottlecollection.sharedCollection().bottleList():
                if self.installTask.bottlename.lower() == bottlename.lower():
                    return True
            else:
                return False


    def bottle_toggle(self, _widget):
        if self.xml.get_widget('BottleSelectContent').get_property('visible'):
            self.advance_section()
        else:
            self.set_expanded_section('BottleSelect')


    def update_bottle_selection_label(self):
        ToggleLabel = self.xml.get_widget("BottleSelectLabel")
        bottlename = self.installTask.bottlename
        if not bottlename:
            ToggleLabel.set_label(_("Select a bottle into which to install"))
        elif self.installTask.GetCreateNewBottle():
            ToggleLabel.set_label(_("Will install into a new %(template)s bottle '%(name)s'") % {
                'template': self.installTask.newBottleTemplate,
                'name': bottlename})
        else:
            ToggleLabel.set_label(_("Will install into the bottle '%(name)s'") % {'name': bottlename})

    def select_bottle(self, model, path, treeiter, _userdata):
        if self.installTask.GetCreateNewBottle():
            bottle_string = '.' + self.installTask.GetNewBottleTemplate()
        elif self.installTask.bottlename:
            bottle_string = self.installTask.bottlename
        else:
            bottle_string = None

        if model.get_value(treeiter, 0) == bottle_string:
            self.xml.get_widget("BottleSelectionView").set_cursor(path)
            return True # stop iteration
        else:
            return False # continue iteration

    def updateBottleListing(self):
        if self.installTask.profile:
            bottles = self.installTask.bottles.values()
            bottles_completed = 0
            for target_bottle in bottles:
                if target_bottle.has_category():
                    bottles_completed += 1

            if bottles_completed != len(bottles):
                progressbar = self.xml.get_widget("BottleSelectProgress")
                progressbar.set_fraction(float(bottles_completed) / len(bottles))
                self.xml.get_widget("BottleSelectProgress").show()
                self.xml.get_widget("BottleSelectListing").hide()
                return

        self.bottleListModel.clear()

        bottles_by_category = {}

        for target_bottle in self.installTask.bottles.values() + self.installTask.templates.values():
            category = target_bottle.get_category()
            if category in bottles_by_category:
                bottles_by_category[category].append(target_bottle)
            else:
                bottles_by_category[category] = [target_bottle]

        for category in bottles_by_category:
            category_row = self.bottleListModel.append(None, row=('', self.category_markup[category], '', self.category_sort[category]))
            for target_bottle in bottles_by_category[category]:
                if isinstance(target_bottle, installtask.TargetTemplate):
                    name = '.' + target_bottle.template
                    markup = _("(New %(template)s bottle)") % {'template': target_bottle.template}
                    sortkey = '0' + target_bottle.template
                else:
                    name = target_bottle.bottlename
                    markup = target_bottle.bottlename
                    sortkey = '1' + target_bottle.bottlename
                template = target_bottle.template
                self.bottleListModel.append(parent=category_row, row=(name, markup, template, sortkey))

        self.xml.get_widget("BottleSelectionView").expand_all()
        self.bottleListModel.foreach(self.select_bottle, None)

        self.xml.get_widget("BottleSelectProgress").hide()
        self.xml.get_widget("BottleSelectListing").show()


    def bottleCreateChanged(self):
        self.bottleListModel.foreach(self.select_bottle, None)
        if self.installTask.GetCreateNewBottle():
            self.xml.get_widget("NewBottleWidgets").show()
        else:
            self.xml.get_widget("NewBottleWidgets").hide()
        self.update_bottle_selection_label()
        self.update_common_widgets()

    def bottleNameChanged(self):
        self.bottleListModel.foreach(self.select_bottle, None)
        self.update_bottle_selection_label()
        self.update_common_widgets()

    def bottleNewnameChanged(self):
        if self.installTask.newBottleName:
            self.xml.get_widget("NewBottleNameEntry").set_text(self.installTask.newBottleName)
        self.update_bottle_selection_label()
        self.update_common_widgets()

    def bottleTemplateChanged(self):
        self.bottleListModel.foreach(self.select_bottle, None)
        self.update_bottle_selection_label()
        self.update_common_widgets()

    # BottleCollection delegate functions
    def bottleCollectionChanged(self):
        removed_bottles = set(self.installTask.bottles)

        for bottle in bottlecollection.sharedCollection().bottles():
            if bottle.name in removed_bottles:
                removed_bottles.remove(bottle.name)
            else:
                if bottle.status in (bottlewrapper.BottleWrapper.STATUS_INIT, bottlewrapper.BottleWrapper.STATUS_READY) and \
                   bottle.bottle_info_ready:
                    self.installTask.add_bottle(bottle)

        for bottlename in removed_bottles:
            self.installTask.remove_bottle_by_name(bottlename)

        self.updateBottleListing()

    # BottleWrapper delegate functions
    def bottleChanged(self, bottle):
        if bottle.status not in (bottlewrapper.BottleWrapper.STATUS_INIT, bottlewrapper.BottleWrapper.STATUS_READY):
            return
        if bottle.name not in self.installTask.bottles:
            # Bottle was renamed
            self.bottleCollectionChanged()
        if bottle.installed_packages_ready:
            self.installTask.installed_applications_ready(bottle)

    def bottle_selection_changed_gui(self, _widget):
        bottlesWidget = self.xml.get_widget("BottleSelectionView")
        tmodel, titer = bottlesWidget.get_selection().get_selected()
        if titer is None:
            # This can happen while we're rebuilding the bottle list
            return
        selectedBottleName = tmodel.get_value(titer, 0)

        if selectedBottleName == '':
            # For categories, select the first child.
            self.xml.get_widget("BottleSelectionView").set_cursor(tmodel.get_path(tmodel.iter_children(titer)))
        elif selectedBottleName.startswith('.'):
            # A bottle template.
            self.installTask.SetCreateNewBottle(create=True, template=selectedBottleName[1:])
        else:
            self.installTask.bottlename = selectedBottleName

    def bottle_newname_changed_gui(self, _widget):
        self.installTask.SetCreateNewBottle(
            newbottlename=self.xml.get_widget("NewBottleNameEntry").get_text().decode('utf8'))

    def new_bottle_insert_text(self, caller, new_text, position, user_data):
        filtered = cxutils.bottlename_sanitize(new_text)
        if filtered != new_text:
            caller.emit_stop_by_name("insert-text")
            caller.emit("insert-text", filtered, position, user_data)


    def no_section_activated(self):
        if self.installEngine:
            # Install was started already.
            return

        self.update_common_widgets()
    

    #-----------------------------------------------------------------------
    #  Install
    #-----------------------------------------------------------------------


    def doInstall(self):
        self.tasks = {}
        self.tasklist = []
        self.cancelInstallation = False
        self.install_in_progress = True
        self.installEngine = cxaiengine.Engine(self.installTask)

        self.totalTasks = len(self.installEngine.get_sorted_tasks()) 
        self.someTasksCanceled = False
        self.someTasksSkipped = False

        if self.registered_delegate:
            collection = bottlecollection.sharedCollection()
            collection.removeChangeDelegate(self)
            collection.removeBottleChangeDelegate(self)
            self.registered_delegate = False

        self.spoolAvailableTasks()


    def displayInstallationProgressPanel(self):
        self.xml.get_widget("PackageSelectToggle").set_sensitive(False)
        self.xml.get_widget("MediaSelectToggle").set_sensitive(False)
        self.xml.get_widget("BottleSelectToggle").set_sensitive(False)
        self.xml.get_widget("InstallerNext").set_sensitive(False)
        self.xml.get_widget("InstallerEject").set_sensitive(True)
        self.xml.get_widget("InstallerToggle").show()
        label = self.xml.get_widget("InstallerLabel")
        label.set_text(_(u"Installing %s\u2026") % self.installTask.profile.name)
        self.set_expanded_section('Installer')
        self.update_install_notes(False)

    def advance_install_progress(self):
        progbar = self.xml.get_widget("installerProgbar")
        progress = 0.0
        for task in self.tasks.itervalues():
            progress += task.progress
        fraction = progress/self.totalTasks
        progbar.set_fraction(fraction)

        task_to_display = None
        for task in self.tasklist:
            if task.finished or not task.aietask.label:
                continue
            if isinstance(task.aietask, cxaiemedia.AIEDownload):
                if task_to_display is None:
                    task_to_display = task
            else:
                task_to_display = task
                break

        if task_to_display:
            progbar.set_text(_(u"%s\u2026") % task_to_display.aietask.label)

    def continueInstallation(self):
        if not self.install_in_progress:
            return

        if self.cancelInstallation:
            cxlog.log("Canceled Installation")

            self.install_in_progress = False

            self.displayInstallationFinishedPanel(True)
        elif self.installEngine.all_done():
            cxlog.log("Installation finished")

            self.install_in_progress = False

            self.displayInstallationFinishedPanel(False)
        else:
            self.spoolAvailableTasks()

    def scroll_tasks(self):
        tasksScroll = self.xml.get_widget("InstallerTasksScroll")
        adj = tasksScroll.get_vadjustment()
        adj.set_value(adj.upper)

    def spoolAvailableTasks(self):
        # Show a failure dialog if necessary
        if self.failing_tasks and not self.install_ui:
            self.failing_tasks.pop(0).show_failure_dialog()
        # Add all the tasks that are currently runnable to the operation queue.
        for task in sorted(self.installEngine.runnable(), cmp=cxaiebase.tasks_cmp):
            if not self.install_ui or not isinstance(task, interactive_tasks):
                if self.installEngine.schedule(task):
                    self.tasks[task] = InstallTask(task, self)
                    self.tasklist.append(self.tasks[task])
        # Maybe update the progress bar text to show what we're doing
        self.advance_install_progress()

    def add_failing_task(self, task):
        self.failing_tasks.append(task)
        self.continueInstallation()

    def cancel_install(self):
        self.cancelInstallation = True
        self.someTasksCanceled = True
        for task in self.tasks.itervalues():
            task.cancel()
        self.continueInstallation()

    def install_toggle(self, _widget):
        if self.xml.get_widget('InstallerContent').get_property('visible'):
            self.set_expanded_section(None)
        else:
            self.set_expanded_section('Installer')

    def install_section_activated(self):
        pass


    def eject_clicked(self, _widget):
        bottlemanagement.wine_eject(self.installTask.bottlename)


    def displayInstallationFinishedPanel(self, inCanceled):
        if inCanceled:
            titleString = _("Installation Canceled")
        else:
            titleString = _("Installation Complete")

        completionMessage = ""
        if self.someTasksCanceled:
            completionMessage = _("The installation failed.")
        elif self.installTask.installWithSteam:
            completionMessage = _("CrossOver has been successfully configured for '%s'. You will need to finish the installation in Steam.") % self.installTask.profile.name
        elif self.someTasksSkipped:
            completionMessage = _("The installation completed, but some tasks were skipped.")
        else:
            completionMessage = _("The installation completed.")

        if 'logfile' in self.installEngine.state:
            url = self.installEngine.state["logfile"]
        elif 'CX_LOG' in self.installEngine.state["environ"]:
            url = self.installEngine.state["environ"]["CX_LOG"]
        else:
            url = None

        if url:
            # FIXME: We should escape url in case it contains weird characters
            # such as '<', '>', '"', etc. Otherwise we will get HTML errors.
            captionHTML = _("<body> <p> %(completion)s </p> <p> More diagnostic information can be found in the <a href=\"file://%(url)s\">debug log</a>. </p> </body>") % {'completion': completionMessage, 'url': url}
        else:
            captionHTML = "<body> <p> %s </p> </body>" % completionMessage

        try:
            #  Encode description as utf8 because htmltextview likes that better.
            caption = self.xml.get_widget("InstallResults")
            caption.display_html(captionHTML.encode('utf8'))
        except:
            cxlog.log("Display_html is failing to render the string %s. Probably a malformed string is being displayed." % cxlog.debug_str(captionHTML))
            caption.get_buffer().set_text(captionHTML)

        self.xml.get_widget("FinishedLabel").set_text(titleString)
        self.xml.get_widget("FinishedToggle").show()
        self.xml.get_widget("InstallerCancel").hide()
        self.set_expanded_section("Finished")

        nextButton = self.xml.get_widget("InstallerNext")
        nextButton.set_sensitive(True)
        stock_close = gtk.stock_lookup(gtk.STOCK_CLOSE)
        if stock_close:
            _stock_id, label, _modifier, _keyval, _translation_domain = stock_close
            nextButton.set_label(label)

        self.xml.get_widget("InstallerEject").set_sensitive(False)

        self.xml.get_widget("installerProgbar").set_text('')

        try:
            # Touch the config file so the bottle manager knows that
            #  something is happening.
            if self.installTask.createNewBottle:
                os.utime(bottlequery.config_file_path(self.installTask.newBottleName), None)
            else:
                os.utime(bottlequery.config_file_path(self.installTask.existingBottleName), None)
        except bottlequery.NotFoundError:
            # The bottle wasn't created yet. Do nothing.
            pass

        if not inCanceled and not self.someTasksCanceled:
            postinstallurl = self.installTask.get_installer_profile().post_install_url
            if postinstallurl:
                cxutils.launch_url(postinstallurl)

    def finished_toggle(self, _widget):
        if self.xml.get_widget('FinishedContent').get_property('visible'):
            self.set_expanded_section(None)
        else:
            self.set_expanded_section('Finished')

    def finished_section_activated(self):
        pass


    #Used to update global state (such as the "Proceed" button name/sensitivity)
    # when the active section changes.
    on_section_changed = {
        'PackageSelect': update_common_widgets,
        'MediaSelect': update_common_widgets,
        'BottleSelect': update_common_widgets,
        'Installer': install_section_activated,
        'Finished': finished_section_activated,
        None: no_section_activated,
    }

    def show_help(self, _caller):
        cxguitools.show_help("install-windows.html")

    def animate(self):
        for entry in self.animateFunctions:
            function, data = entry
            if data:
                function(data)
            else:
                function()
        if self.install_in_progress:
            self.advance_install_progress()
        return True

    def addAnimationFunction(self, inFunction, userData=None):
        self.animateFunctions.append([inFunction, userData])

    def removeAnimationFunction(self, inFunction, userData=None):
        if [inFunction, userData] in self.animateFunctions:
            self.animateFunctions.remove([inFunction, userData])

def update_progress_bar(inData):
    task, progbar = inData
    if isinstance(task, cxaiemedia.AIEDownload):
        if task.size == -1:
            progbar.set_text(_(u"Preparing to download\u2026"))
            progbar.pulse()
        elif task.size == 0:
            progbar.set_text(_("(unknown size)"))
            progbar.pulse()
        else:
            def size_to_str(size):
                if size < 2 * 1024 * 1024:
                    return _("%dkB") % (size / 1024)
                else:
                    return _("%.1fMB") % (size / 1024.0 / 1024)

            progbar.set_text(_("Downloaded %(part)s of %(total)s") % {'part':size_to_str(task.downloaded), 'total':size_to_str(task.size)})
            progbar.set_fraction(task.progress)

interactive_tasks = (cxaiecore.AIECore, )

class InstallTask(object):
    # widgets
    icon = None
    caption = None
    progbar = None
    frame = None
    cancel_button = None
    finished = False

    def __init__(self, aietask, wizard):
        self.aietask = aietask
        self.wizard = wizard

        if aietask.label:
            tasksBox = self.wizard.xml.get_widget("InstallerTasksBox")
            taskFrame = gtk.VBox()
            taskBox = gtk.HBox()

            progbar = gtk.ProgressBar()
            if isinstance(aietask, cxaiemedia.AIEDownload):
                icon = gtk.image_new_from_stock(gtk.STOCK_COPY, gtk.ICON_SIZE_DND)
                progbar.set_pulse_step(0.001)
                self.wizard.addAnimationFunction(update_progress_bar, [aietask, progbar])
            elif isinstance(aietask, cxaiemedia.AIEScanMedia):
                icon = gtk.image_new_from_stock(gtk.STOCK_FIND, gtk.ICON_SIZE_DND)
                progbar.set_pulse_step(0.02)
                self.wizard.addAnimationFunction(progbar.pulse)
            else:
                icon = gtk.image_new_from_stock(gtk.STOCK_EXECUTE, gtk.ICON_SIZE_DND)
                progbar.set_pulse_step(0.03)
                self.wizard.addAnimationFunction(progbar.pulse)
            taskBox.pack_start(icon, False, False, 2)
            self.icon = icon

            progBox = gtk.VBox()
            label1 = gtk.Label(_(u"%s\u2026") % aietask.label)
            label1.set_alignment(0, 0.5)
            self.caption = label1
            progBox.pack_start(label1, False, False, 2)
            self.progbar = progbar

            progBox.pack_start(progbar, False, True, 2)
            taskBox.pack_start(progBox, True, True, 10)

            self.cancel_button = gtk.Button(stock=gtk.STOCK_CANCEL)
            self.cancel_button.connect('clicked', self.cancel_clicked)
            cancel_vbox = gtk.VBox()
            cancel_vbox.pack_start(gtk.Label())
            cancel_vbox.pack_start(self.cancel_button, False, False)
            cancel_vbox.pack_start(gtk.Label())
            taskBox.pack_start(cancel_vbox, False, False)

            self.frame = taskFrame
            taskFrame.pack_start(taskBox, False, False, 2)
            taskFrame.pack_start(gtk.HSeparator(), False, False, 2)
            tasksBox.pack_start(taskFrame, False, False, 0)
            taskFrame.show_all()
            tasksBox.show()

            self.wizard.scroll_tasks()

        self.enqueue()

    def get_progress(self):
        if self.finished:
            return 1.0
        elif isinstance(self.aietask, cxaiemedia.AIEDownload):
            return self.aietask.progress
        else:
            return 0.0

    progress = property(get_progress)

    def enqueue(self):
        if isinstance(self.aietask, interactive_tasks):
            self.wizard.install_ui = True
        self.operation = InstallTaskOperation(self)
        if isinstance(self.aietask, cxaiemedia.AIEDownload):
            download_queue.enqueue(self.operation)
        else:
            pyop.sharedOperationQueue.enqueue(self.operation)

    def done(self):
        # This task has either succeeded or failed and will not retry. Update
        # widgets and notify the ai engine.
        if self.progbar:
            self.progbar.hide()
        if self.cancel_button:
            self.cancel_button.hide()
        if isinstance(self.aietask, interactive_tasks):
            self.wizard.install_ui = False

        self.finished = True
        self.wizard.scroll_tasks()
        self.aietask.done()
        self.wizard.advance_install_progress()
        self.wizard.continueInstallation()

    def task_returned(self, success):
        if self.finished:
            return
        if isinstance(self.aietask, interactive_tasks):
            self.wizard.install_ui = False
        self.operation = None
        if success:
            if self.caption:
                self.caption.set_text(self.caption.get_text() + " " + _("Done."))
            self.done()
        else:
            if self.caption:
                self.caption.set_text(self.caption.get_text() + " " + _("Failed."))

            self.wizard.add_failing_task(self)

    def show_failure_dialog(self):
        if isinstance(self.aietask, cxaiemedia.AIEDownload):
            primary = _("Could not download the %(appname)s installer") % {'appname': self.aietask.aiecore.name}
        elif isinstance(self.aietask, cxaiecore.AIECore):
            primary = _("An error occurred while installing %(appname)s") % {'appname': self.aietask.aiecore.name}
        else:
            label = self.aietask.label
            if not label:
                label = unicode(self.aietask)
            primary = _("An error occurred while running the '%(task)s' task") % {'task': label}

        if self.aietask.error:
            secondary = self.aietask.error
        else:
            secondary = _("An unexpected error occurred.")
        if hasattr(self.aietask, "needs_download") and self.aietask.needs_download and hasattr(self.aietask, "url"):
            secondary += _("\n\nIf you have a working Internet connection via your browser, you can download this file here:\n\n<a href=\"%(url)s\">%(url)s</a>") % {'url': self.aietask.url}

        message = '<span weight="bold" size="larger">%s</span>\n\n%s' % (primary, secondary)

        panel = self.wizard.xml.get_widget("InstallationProgress")
        buttons = [[_("Try Again"), 1]]
        if isinstance(self.aietask, cxaiemedia.AIEDownload):
            buttons.append([_("Pick Installer File"), 3])
        buttons.extend(([_("Skip This Step"), 0],
                        [_("Cancel Installation"), 2]))
        cxguitools.CXMessageDlg(markup=message,
                                button_array=buttons,
                                response_function=self.abortRetryFail,
                                parent=panel,
                                message_type=gtk.MESSAGE_WARNING)

        self.wizard.install_ui = True

    def skip(self):
        self.wizard.someTasksSkipped = True
        self.done()

    def retry(self):
        if self.caption:
            self.caption.set_text(self.caption.get_text() + u' ' + _(u"Retrying\u2026"))
        self.enqueue()

    def cancel(self):
        if self.finished:
            return
        if self.aietask.can_cancel():
            self.aietask.cancel()
        if self.caption:
            self.caption.set_text(self.caption.get_text() + u' ' + _("Canceled."))
        self.done()

    def abortRetryFail(self, response):
        self.wizard.install_ui = False
        if response == 0:
            self.skip()
        elif response == 1:
            self.retry()
        elif response == 2:
            self.wizard.cancel_install()
        elif response == 3:
            panel = self.wizard.xml.get_widget("InstallationProgress")
            filePicker = gtk.FileChooserDialog(_("Please locate the installer file for %s") % self.aietask.aiecore.name, panel, gtk.FILE_CHOOSER_ACTION_OPEN)

            cxguitools.add_filters(filePicker, cxguitools.FILTERS_INSTALLABLE)
            filePicker.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
            filePicker.add_button(gtk.STOCK_OPEN, gtk.RESPONSE_OK)

            if filePicker.run() == gtk.RESPONSE_OK:
                self.aietask.aiecore.state['install_source'] = filePicker.get_filename()
                self.retry()
            else:
                # Go back to the previous error dialog
                self.show_failure_dialog()
            filePicker.destroy()
            return

        # Just in case we need to show another failure dialog
        self.wizard.continueInstallation()


    def cancel_clicked(self, _widget):
        self.cancel()

class UpdateProfilesOperation(pyop.PythonOperation):

    def __init__(self, url, wizard):
        pyop.PythonOperation.__init__(self)
        self.url = url
        self.wizard = wizard

    def __unicode__(self):
        return "%s - %s" % (self.__class__.__name__, unicode(self.url))

    def main(self):
        c4profilesmanager.update_online_profiles(self.url)
        self.wizard.installTask.update_profiles()

    def enqueued(self):
        progbar = self.wizard.xml.get_widget('PackageUpdateBar')
        progbar.show()
        self.wizard.addAnimationFunction(progbar.pulse)

    def finish(self):
        progbar = self.wizard.xml.get_widget('PackageUpdateBar')
        self.wizard.removeAnimationFunction(progbar.pulse)
        if self.wizard.xml.get_widget('InstallerAssistant').get_property('visible'):
            self.wizard.updatePackageListing()
        self.wizard.xml.get_widget('PackageUpdateBar').hide()
        self.wizard.profileChanged()
        self.wizard.package_changed_gui(None)

class InstallTaskOperation(pyop.PythonOperation):

    def __init__(self, task):
        pyop.PythonOperation.__init__(self)
        self.task = task
        self.success = False

    def __unicode__(self):
        return "%s - %s" % (self.__class__.__name__, unicode(self.task))

    def main(self):
        if not self.task.wizard.cancelInstallation and not self.task.finished:
            self.success = self.task.aietask.main()
        else:
            self.success = False

    def finish(self):
        self.task.task_returned(self.success)

